#include <iostream>
#include <fstream>
#include <string.h>
#include <string>
#include <list>
#include <map>
#include <File.h>
#include <stdio.h>
#include <Directory.h>
#include <Path.h>
#include <NodeInfo.h>
#include <algorithm>
#include <vector>
#include <StopWatch.h>
#include <Application.h>
#include "FinderWindow.h"
#include <Message.h>
#include <strstream>

#include "libpq++.h"

#include "utils.h"

#include "Finder.h"

//#define DEBUG 

void
find_word( PgDatabase & data, map<string,double> & results, string word )
{
	#ifdef DEBUG
	cout << "Searching for " << word << endl;
	#endif
	
	BStopWatch timer("word search");
	
	string query;
	query = 
		string("SELECT fileTable.path,contentTable.weight ") +
		string("FROM fileTable,contentTable,wordTable ") + 
		string("WHERE ") +
			string("wordTable.word='" + word + "' AND ") +
			string("wordTable.id=contentTable.word AND ") +
			string("fileTable.id=contentTable.file ");
		string("ORDER BY contentTable.weight;");
	
	do_query( data, query.c_str() );
	
	for ( int i=0; i<data.Tuples(); i++ )
	{
		results[data.GetValue(i,"path")] = atof(data.GetValue(i,"weight"));
	}
}

void
find_words( PgDatabase & data, map<string,double> & matches, list<string> & words )
{
	BStopWatch timer("database search");
	
	list<string>::iterator curr = words.begin();
	
	if ( curr == words.end() )
		return;
	
	find_word(data,matches,*curr);
	
	curr++;
	
	// search for all words, but stop if we have no more matches
	while ( curr != words.end() && matches.size() > 0 )
	{
		map<string,double> results;
		
		find_word(data,results,*curr);
		
		// now, remove paths in matches that aren't in results
		// and update weight if they are
		map<string,double>::iterator iter = matches.begin();
		
		while ( iter != matches.end() )
		{
			if ( results.find(iter->first) == results.end() )
			{
				// not a match?
				#ifdef DEBUG
				cout << "removing " << iter->first << endl;
				#endif
				matches.erase(iter++);
			} else 
			{
				matches[iter->first] = matches[iter->first] + iter->second;
				iter++;
			}
		}
		
		curr++;
	}
}

void
search( map<string,double> & matches, list<string> & words )
{
	PgDatabase data("dbname=beindexed host=127.0.0.1");
	
	find_words(data,matches,words);
	
	// sort matches by weight instead of by path
	vector<pair<double,string> > result;
	
	map<string,double>::iterator iter = matches.begin();
	
	while ( iter != matches.end() )
	{
		result.push_back( pair<double,string>(iter->second,iter->first) );
		iter++;
	}
	
	// we actually want to reverse_sort(being,end), but..
	sort(result.begin(),result.end());
	reverse(result.begin(),result.end());
}

typedef struct bg_search_data_t {
	BMessenger	msgr;
	string		query;
};

int32
background_search_thread( void * _data )
{
	bg_search_data_t * data = (bg_search_data_t*)_data;
	
	// split query
	list<string> words;
	
	istrstream i( data->query.c_str() );
	
	while ( !i.eof() )
	{
		skip_non_text(i);
		string word = toLower( get_word(i) );
		
		if ( word == "" )
			// we get this at eof
			continue;
		
		words.push_back(word);
	}
	
	// hackish bug fix: remove last character in last word. this shouldn't be needed
	string last_word = (*(--words.end()));
	words.pop_back();
	last_word.erase( last_word.length()-1, 1 );
	words.push_back( last_word );
	
	map<string,double> matches;
	
	// search
	if ( words.size() > 0 )
		search(matches,words);
	
	BMessage msg(FINDER_RESULTS),reply;
	
	// foreach match add to msg
	for ( map<string,double>::iterator i = matches.begin(); i != matches.end(); i++ )
	{
		msg.AddString("path", (*i).first.c_str());
		msg.AddDouble("weight", (*i).second);
	}
	
	data->msgr.SendMessage(&msg,&reply);
	
	delete data;
	
	return 0;
}

void
background_search( BMessenger msgr, string query_string )
{
	bg_search_data_t * data = new bg_search_data_t;
	
	data->msgr = msgr;
	data->query = query_string;
	
	thread_id t = spawn_thread(
		background_search_thread,
		"Searcher",
		B_NORMAL_PRIORITY,
		data
	);
	
	resume_thread(t);
}


int 
main(int numArg, const char ** argv)
{
/*	list<string> words;
	
	for ( int i=1; i<numArg; i++ )
		words.push_back( toLower(argv[i]) );
*/
	BApplication app("application/x-vnd.m_eiman.Finder");
	
	FinderWindow * win = new FinderWindow;
	win->Show();
	
	app.Run();
	
	return 0;
}
