#include "FinderWindow.h"

#include <Application.h>

#include "HGroup.h"
#include "VGroup.h"
#include "MBorder.h"
#include "MScrollView.h"

#include "Finder.h"

#include <ListItem.h>
#include <Entry.h>
#include <strstream>

FinderWindow::FinderWindow()
:	MWindow(
		BRect(100,100,500,400),
		"Finder",
		B_TITLED_WINDOW,
		B_ASYNCHRONOUS_CONTROLS|B_QUIT_ON_WINDOW_CLOSE
	)
{
	BView * view =
	new VGroup(
		new MBorder(
			B_FANCY_BORDER,
			5,
			NULL,
			m_text = new MTextControl(
				"Search", ""
			)
		),
		new MScrollView(
			m_list = new MListView(),
			false,
			true
		),
		m_status = new MStringView(
			"Enter search terms and press Enter to search."
		),
		NULL
	);
	
	AddChild( view );

	m_text->MakeFocus();
	m_text->SetMessage( new BMessage(SEARCH_BUTTON) );
	m_list->SetMessage( new BMessage(LIST) );
}

FinderWindow::~FinderWindow()
{
}

bool
FinderWindow::QuitRequested()
{
	be_app->PostMessage( B_QUIT_REQUESTED );
	return true;
}

void
FinderWindow::MessageReceived( BMessage * msg )
{
	switch ( msg->what )
	{
		case LIST:
		{
			int32 offset=-1;
			if ( msg->FindInt32("index",&offset) == B_OK )
			{
				BStringItem * item = (BStringItem*)m_list->ItemAt(offset);
				entry_ref ref;
				get_ref_for_path(item->Text(),&ref);
				
				BMessage message(B_REFS_RECEIVED);
				message.AddRef("refs", &ref);
				BMessenger trackerMsgr( "application/x-vnd.Be-TRAK" );
				trackerMsgr.SendMessage(&message);
			}
		}	break;
		case SEARCH_BUTTON:
			m_status->SetText("Searching...");
			background_search( BMessenger(this), m_text->Text() );
			break;
		case FINDER_RESULTS:
		{
			// clear list
			while ( m_list->ItemAt(0) )
			{
				BListItem * item = m_list->ItemAt(0);
				m_list->RemoveItem(item);
				delete item;
			}
			
			// add new results
			int count=0;
			const char * path;
			for ( count=0; msg->FindString("path",count,&path) == B_OK; count++ )
			{
				m_list->AddItem( new BStringItem(path) );
			}
			
			strstream str;
			str << "Found " << count << " matching files. "
				<< "Enter new search terms and press Enter to search again.";
			
			char status[1024];
			str.getline(status,sizeof(status)-1);
				
			m_status->SetText( status );
		}	break;
		default:
			MWindow::MessageReceived(msg);
	}
}
